#ifndef IN_EQUIS_ARRAY2_HPP
#error This file must be included via "array2.hpp"
#endif

EQUIS_ENTER_NS1(equis);

template < typename T > inline
array2<T>::array2()
    : m_ptr(NULL), m_rows(0), m_cols(0), m_free_ptr(false)
{
}

template < typename T > inline
array2<T>::array2(T* ptr, long rows, long cols, bool free_ptr)
    : m_ptr(ptr), m_rows(rows), m_cols(cols), m_free_ptr(free_ptr)
{}

template < typename T > inline
array2<T>::~array2() {
    if (m_ptr && m_free_ptr) {
        _aligned_free(m_ptr);
    }
}

template < typename T > inline
void
array2<T>::clear(const T& value) {
	for (long i = 0; i < size(); ++i) {
		m_ptr[i] = value;
	}
}

template < typename T > inline
bool
array2<T>::resize(long rows, long cols, const T& value) {
    if (rows * cols == m_rows * m_cols) return true;

	long size = rows * cols;
	T* new_ptr = reinterpret_cast<T*>(
		_aligned_malloc(size * sizeof(T), 16));
	if (!new_ptr) return false;

	for (long i = 0; i < rows * cols; ++i) {
		new (&new_ptr[i]) T(value);
	}

	m_ptr = new_ptr;
	m_rows = rows;
	m_cols = cols;
	m_free_ptr = true;
	return true;
}

template < typename T > inline
bool array2<T>::empty() const {
	return m_rows == 0 && m_cols == 0;
}

template < typename T > inline
long
array2<T>::rows() const {
	return m_rows;
}

template < typename T > inline
long
array2<T>::cols() const {
	return m_cols;
}

template < typename T > inline
long
array2<T>::size() const {
	return m_rows * m_cols;
}

template < typename T > inline
bool
array2<T>::valid(long row, long col) const {
	return row >= 0 && row < m_rows && col >= 0 && col < m_cols;
}

template < typename T > inline
T&
array2<T>::operator [] (long index) {
	EQUIS_DEBUG_ASSERT(valid(index));
	return m_ptr[index];
}

template < typename T > inline
const T&
array2<T>::operator [] (long index) const {
	EQUIS_DEBUG_ASSERT(valid(index));
	return m_ptr[index];
}

template < typename T > inline
T&
array2<T>::operator () (long row, long col) {
	return at(row, col);
}

template < typename T > inline
const T&
array2<T>::operator () (long row, long col) const {
	return at(row, col);
}

template < typename T > inline
T&
array2<T>::at(long row, long col) {
	EQUIS_DEBUG_ASSERT(valid(row, col));
	return m_ptr[row * m_cols + col];
}

template < typename T > inline
const T&
array2<T>::at(long row, long col) const {
	EQUIS_DEBUG_ASSERT(valid(row, col));
	return m_ptr[row * m_cols + col];
}

template < typename T > inline
typename array2<T>::iterator
array2<T>::begin() {
	return &m_ptr[0];
}

template < typename T > inline
typename array2<T>::iterator
array2<T>::end() {
	return &m_ptr[size()];
}

template < typename T > inline
typename array2<T>::const_iterator
array2<T>::begin() const {
	return &m_ptr[0];
}

template < typename T > inline
typename array2<T>::const_iterator
array2<T>::end() const {
	return &m_ptr[size()];
}

template < typename T > inline
typename array2<T>::iterator
array2<T>::row(long index) {
	EQUIS_DEBUG_ASSERT(index >= 0 && index < m_rows);
	return &m_ptr[index * m_cols];
}

template < typename T > inline
typename array2<T>::const_iterator
array2<T>::row(long index) const {
	EQUIS_DEBUG_ASSERT(index >= 0 && index < m_rows);
	return &m_ptr[index * m_cols];
}

EQUIS_LEAVE_NS1(equis);
