#ifndef IN_EQUIS_IMAGE_PIXEL_HPP
#error This file must be included via pixel.hpp
#endif /* IN_EQUIS_IMAGE_PIXEL_HPP */

EQUIS_ENTER_NS2(equis,image);

//*** Conversions to PixelA8R8G8B8

template <> inline
void convertPixel<PixelA8R8G8B8, PixelA8R8G8B8>(PixelA8R8G8B8& dst, const PixelA8R8G8B8& src) {
	dst.a = src.a;
	dst.r = src.r;
	dst.g = src.g;
	dst.b = src.b;
}

template <> inline
void convertPixel<PixelA8R8G8B8, PixelR8G8B8>(PixelA8R8G8B8& dst, const PixelR8G8B8& src) {
	dst.a = 0xFF;
	dst.r = src.r;
	dst.g = src.g;
	dst.b = src.b;
}

template <> inline
void convertPixel<PixelA8R8G8B8, PixelY8>(PixelA8R8G8B8& dst, const PixelY8& src) {
	dst.a = 0xFF;
	dst.r = dst.g = dst.b = src.y;
}

template <> inline
void convertPixel<PixelA8R8G8B8, PixelY16>(PixelA8R8G8B8& dst, const PixelY16& src) {
	dst.a = 0xFF;
	dst.r = dst.g = dst.b = src.y >> 8;
}

template <> inline
void convertPixel<PixelA8R8G8B8, PixelY8U8V8>(PixelA8R8G8B8& dst, const PixelY8U8V8& src) {
	const int y = src.y - 16;
	const int u = src.u - 128;
	const int v = src.v - 128;
	
	int r = (298 * y           + 409 * v + 128) >> 8;
	int g = (298 * y - 100 * u - 208 * v + 128) >> 8;
	int b = (298 * y + 516 * u           + 128) >> 8;

	if (r < 0) r = 0; else if (r > 255) r = 255;
	if (g < 0) g = 0; else if (g > 255) g = 255;
	if (b < 0) b = 0; else if (b > 255) b = 255;

	dst.a = 0xFF;
	dst.r = r;
	dst.g = g;
	dst.b = b;
}

//*** Conversions to PixelY8

// Most of the computation getting from RGB to luminance involves
// converting the sum of color components from 0..765 back to 0..255
//   Y = (R + G + B) / 3
//
// Multiply through by 256
//   256 * Y = (R + G + B) * 85.33333
//
// Distribute, with the green component weighted more heavily
//   256 * Y = 85 * R + 86 * G + 85 * B

template <> inline
void convertPixel<PixelY8, PixelA8R8G8B8>(PixelY8& dst, const PixelA8R8G8B8& src) {
	dst.y = (85 * src.r + 86 * src.g + 85 * src.b) >> 8;
}

template <> inline
void convertPixel<PixelY8, PixelR8G8B8>(PixelY8& dst, const PixelR8G8B8& src) {
	dst.y = (85 * src.r + 86 * src.g + 85 * src.b) >> 8;
}

template <> inline
void convertPixel<PixelY8, PixelY8>(PixelY8& dst, const PixelY8& src) {
	dst.y = src.y;
}

template <> inline
void convertPixel<PixelY8, PixelY16>(PixelY8& dst, const PixelY16& src) {
	dst.y = src.y >> 8;
}

template <> inline
void convertPixel<PixelY8, PixelY8U8V8>(PixelY8& dst, const PixelY8U8V8& src) {
	dst.y = src.y;
}

//*** Conversions to PixelY16

template <> inline
void convertPixel<PixelY16, PixelA8R8G8B8>(PixelY16& dst, const PixelA8R8G8B8& src) {
	dst.y = 85 * src.r + 86 * src.g + 85 * src.b;
}

template <> inline
void convertPixel<PixelY16, PixelR8G8B8>(PixelY16& dst, const PixelR8G8B8& src) {
	dst.y = 85 * src.r + 86 * src.g + 85 * src.b;
}

template <> inline
void convertPixel<PixelY16, PixelY8>(PixelY16& dst, const PixelY8& src) {
	dst.y = src.y << 8;
}

template <> inline
void convertPixel<PixelY16, PixelY16>(PixelY16& dst, const PixelY16& src) {
	dst.y = src.y;
}

template <> inline
void convertPixel<PixelY16, PixelY8U8V8>(PixelY16& dst, const PixelY8U8V8& src) {
	dst.y = src.y << 8;
}

EQUIS_LEAVE_NS2(equis,image);