#ifndef IN_EQUIS_WIN32_BASEUNKNOWN_HPP
#error This file must be included via BaseUnknown.hpp
#endif

EQUIS_ENTER_NS1(equis);

inline
BaseUnknown::BaseUnknown()
  : m_referenceCount(1)
{}

inline STDMETHODIMP
BaseUnknown::QueryInterface(REFIID iid, void** result)
{
    if (result) *result = NULL;

    HRESULT res = S_OK;

    if (result == NULL) {
        res = E_POINTER;
    } else if (IsEqualIID(iid, IID_IUnknown)) {
        *result = implicit_cast<IUnknown*>(this);
        AddRef();
    } else {
        res = E_NOINTERFACE;
    }

    return res;
}

inline STDMETHODIMP_(ULONG)
BaseUnknown::AddRef()
{
    LONG referenceCount = InterlockedIncrement(&m_referenceCount);
    return (ULONG)referenceCount;
}

inline STDMETHODIMP_(ULONG)
BaseUnknown::Release()
{
    LONG referenceCount = InterlockedDecrement(&m_referenceCount);
    if (referenceCount == 0) {
        delete this;
    }
    return (ULONG)referenceCount;
}

EQUIS_LEAVE_NS1(equis);
