#ifndef EQUIS_WIN32_MEDIATYPE_HPP
#define EQUIS_WIN32_MEDIATYPE_HPP

EQUIS_ENTER_NS1(equis);

/// A safe media type pointer.
class MediaTypePtr {
public:
    /// Creates a NULL pointer.
    MediaTypePtr();

    /// Detaches this pointer.
    ~MediaTypePtr();

    /// Determines if this pointer is NULL.
    bool IsNull() const;

    /// Determines if this pointer is not NULL.
    bool IsNotNull() const;

    /// Directly accesses the underlying pointer. In debug mode, invoking
    /// this operator on a NULL pointer will trigger a breakpoint.
    AM_MEDIA_TYPE* operator -> ();

	/// Directly accesses the underlying pointer. In debug mode, invoking
    /// this operator on a NULL pointer will trigger a breakpoint.
    const AM_MEDIA_TYPE* operator -> () const;

    /// Gets the underlying pointer.
    operator AM_MEDIA_TYPE* ();

	/// Gets the underlying pointer.
    operator const AM_MEDIA_TYPE* () const;

    /// Attaches to the specified pointer.
    void Set(AM_MEDIA_TYPE* ptr);

    /// Attaches to the specified pointer without releasing it later.
    void SetWithoutRelease(AM_MEDIA_TYPE* ptr);

	/// Clones the specified pointer.
	void Clone(AM_MEDIA_TYPE* ptr);

    /// Detaches this pointer, and sets it to NULL.
    void Clear();

    /// Detaches this pointer without releasing it, and sets it to NULL.
    AM_MEDIA_TYPE* ClearWithoutRelease();

    /// Detaches this pointer, sets it to NULL, and returns its address.
    AM_MEDIA_TYPE** ClearAndGetPointer();

private:
    AM_MEDIA_TYPE* m_ptr;
    bool m_noRelease;
};

EQUIS_LEAVE_NS1(equis);

#define IN_EQUIS_WIN32_MEDIATYPE_HPP
#include "MediaType.inl"
#undef IN_EQUIS_WIN32_MEDIATYPE_HPP

#endif /* EQUIS_WIN32_MEDIATYPE_HPP */