#include "common.hpp"
EQUIS_USING_NS1(equis);

Mutex::Mutex(HRESULT& res)
	: m_handle(NULL)
{
    if (res == S_OK) {
        m_handle = CreateMutex(NULL, FALSE, NULL);
        if (m_handle == NULL) {
            res = HRESULT_FROM_WIN32(GetLastError());
        }
    }
}

Mutex::~Mutex() {
    HRESULT res = S_OK;

    if (m_handle != NULL) {
        if (!CloseHandle(m_handle)) {
            res = HRESULT_FROM_WIN32(GetLastError());
        }
        m_handle = NULL;
    }

    _ASSERT(SUCCEEDED(res));
}

HRESULT
Mutex::Enter(DWORD timeoutMs) {
    HRESULT res = S_OK;

    if (m_handle == NULL) {
        res = E_FAIL;
    } else {
        DWORD waitResult = WaitForSingleObject(m_handle, timeoutMs);

        if (waitResult == WAIT_FAILED) {
            res = HRESULT_FROM_WIN32(GetLastError());
        } else if (waitResult == WAIT_TIMEOUT) {
            res = E_ABORT;
        } else if (waitResult != WAIT_OBJECT_0 && waitResult != WAIT_ABANDONED) {
            res = E_FAIL;
        }
    }

    // Will occasionally encounter a timeout during system shutdown.
    _ASSERT(SUCCEEDED(res) || res == E_ABORT);
    return res;
}

HRESULT
Mutex::Leave() {
    HRESULT res = S_OK;

    if (m_handle == NULL) {
        res = E_FAIL;
    }
    else {
        if (!ReleaseMutex(m_handle)) {
            res = HRESULT_FROM_WIN32(GetLastError());
        }
    }

    _ASSERT(SUCCEEDED(res));
    return res;
}
