using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Demos.Rocks
{
    public static class GameTools
    {
        private static Random random = new Random();

        public static float Cos(float th)
        {
            return (float) Math.Cos(th);
        }

        public static float Sin(float th)
        {
            return (float) Math.Sin(th);
        }

        public static int Square(int x)
        {
            return x * x;
        }

        public static float Square(float x)
        {
            return x * x;
        }

        public static int RandomInt(int min, int max)
        {
            return random.Next(Math.Max(0, max - min)) + min;
        }

        public static float RandomFloat(float min, float max)
        {
            return (float)random.NextDouble() * (max - min) + min;
        }

        public static Vector2 RandomVector2(float minLength, float maxLength)
        {
            float length = RandomFloat(minLength, maxLength);
            float theta = RandomFloat(0.0f, MathHelper.TwoPi);

            return new Vector2(
                length * Cos(theta),
                length * Sin(theta));
        }

        public static Vector3 RandomVector3()
        {
            return RandomVector3(1.0f, 1.0f);
        }

        public static Vector3 RandomVector3(float minLength, float maxLength)
        {
            float length = RandomFloat(minLength, maxLength);
            float theta = RandomFloat(0.0f, MathHelper.Pi);
            float phi = RandomFloat(0.0f, MathHelper.TwoPi);

            return new Vector3(
                length * Cos(phi) * Cos(theta),
                length * Sin(phi) * Cos(theta),
                length * Sin(theta));
        }

        public static Vector3 UnprojectToZeroPlane(RocksGame game, Vector2 point)
        {
            Viewport viewport = game.GraphicsManager.GraphicsDevice.Viewport;

            Vector3 near = viewport.Unproject(
                new Vector3(point, viewport.MinDepth),
                game.Projection, game.View, Matrix.Identity);

            Vector3 far = viewport.Unproject(
                new Vector3(point, viewport.MaxDepth),
                game.Projection, game.View, Matrix.Identity);

            float amount = near.Z / (near.Z - far.Z);
            return Vector3.Lerp(near, far, amount);
        }
    }
}
