#ifndef EQUIS_IMAGE_BLOB_TRACK_HH
#define EQUIS_IMAGE_BLOB_TRACK_HH

#include <set>
#include <vector>

#include "touch.h"
#include "blob.hpp"

class BlobTracker {
public:
    BlobTracker();
    ~BlobTracker();
    
    long GetMinAge();
    void SetMinAge(long value);

    long GetMaxAge();
    void SetMaxAge(long value);

    void Resize(long rows, long cols);
    void Update(const vector<Blob>& blobs);

    struct Event {
        long type;
        long frame;
		long since;
        Blob blob;

        Event(long type, long frame, long since, Blob blob)
            : type(type), frame(frame), since(since), blob(blob) {}
    };

    void Poll(vector<Event>& events);
    long Poll(ETEvent* events, long events_cap);

private:
    long m_frame;
	long m_rows;
	long m_cols;

    long m_next_session_id;
	long m_next_cursor_id;
	std::set<long> m_free_cursor_ids;

    long m_min_age;
    long m_max_age;
    
    struct BlobI : public Blob {
		long first_frame;
		long last_frame;

        BlobI(const Blob& source);
    };

    struct Cell {
        vector<BlobI> blobs;
    };
    
    long m_step;
	equis::array2<Cell> m_cells;
	std::vector<Event> m_events;

    BlobI* FindBestMatch(const BlobI& curr);
    float MatchPenalty(const BlobI& curr, const BlobI& prev);

	long create_cursor_id();
	void release_cursor_id(long cursor_id);
};

#endif /* EQUIS_IMAGE_BLOB_TRACK_HH */
