#ifndef EQUIS_TOUCH_H
#define EQUIS_TOUCH_H

#include "video/input.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct ETSurface ETSurface;
typedef struct ETEvent ETEvent;

struct ETEvent {
	long frame;
	long since;
	long type;
	long cursor_id;
	long session_id;
	float x;
	float y;
	float mass;
};

#define ET_TYPE_PRESS (0)
#define ET_TYPE_UPDATE (1)
#define ET_TYPE_RELEASE (2)

ETSurface* etCreate();

int etDelete(ETSurface* handle);

int etSourceVideoOpen(ETSurface* handle, const char* uri);
int etSourceVideo(ETSurface* handle, EVInput* video, int auto_close);

long etPoll(ETSurface* handle, ETEvent* events, long events_cap);
long etNext(ETSurface* handle, ETEvent* events, long events_cap);

int etCalibrate(ETSurface* handle, const float* desired, const float* reported, long end);

int etGetFormat(ETSurface* handle, EVFormat* format);
long etPreview(ETSurface* handle, int source, void* image, long image_stride, long image_cap);

#define ET_PREVIEW_INPUT (0)
#define ET_PREVIEW_BACKGROUND (1)
#define ET_PREVIEW_FOREGROUND (2)
#define ET_PREVIEW_FILTERED (3)
#define ET_PREVIEW_BLOBS (4)
#define ET_PREVIEW_TOUCHES (5)
#define ET_PREVIEW_EVENTS (6)

/// Reads an integer parameter.
int etGeti(
	ETSurface* handle, int name,
    int* value);

/// Reads a floating-point parameter.
int etGetf(
	ETSurface* handle, int name,
	float* value);

/// Writes an integer parameter.
int etSeti(
	ETSurface* handle, int name,
	int value);

/// Writes a floating-point parameter.
int etSetf(
	ETSurface* handle, int name,
	float value);

#define ET_NOISE_FILTER (1)
#define ET_BACKGROUND_DRIFT (2)
#define ET_BACKGROUND_THRESHOLD (3)
#define ET_PIXEL_THRESHOLD (4)
#define ET_TOUCH_THRESHOLD (5)
#define ET_STICKY_PRESS (6)
#define ET_STICKY_RELEASE (7)

#ifdef __cplusplus
}
#endif

#endif
