#include "common.hpp"
#include "ArrayEnumPins.hpp"
EQUIS_USING_NS1(equis);

ArrayEnumPins::ArrayEnumPins(IPin** pins, ULONG length)
: m_pins(pins), m_length(length), m_index(0)
{}

ArrayEnumPins::~ArrayEnumPins()
{}

STDMETHODIMP ArrayEnumPins::QueryInterface(REFIID iid, void** result) {
    HRESULT res = BaseUnknown::QueryInterface(iid, result);
    if (res == E_NOINTERFACE) {
        if (IsEqualIID(iid, IID_IEnumPins)) {
            *result = implicit_cast<IEnumPins*>(this);
        }
        if (*result) {
            AddRef();
            res = S_OK;
        }
    }
    return res;
}

STDMETHODIMP
ArrayEnumPins::Next(ULONG length, IPin **pins, ULONG *used) {
    HRESULT res = S_OK;

    if (pins == NULL || (length != 1 && used == NULL)) {
        res = E_POINTER;
    } else {
        ULONG position;
        for (position = 0; position < length && m_index + position < m_length;  ++position)
        {
            pins[position] = m_pins[m_index + position];
            if (pins[position]) pins[position]->AddRef();
        }

        m_index += position;
        if (used != NULL) *used = position;
        if (res == S_OK && position < length) res = S_FALSE;
    }

    return res;
}

STDMETHODIMP
ArrayEnumPins::Skip(ULONG length) {
    HRESULT res = S_OK;

    {
        ULONG position = std::max(m_length - m_index, length);

        m_index += position;
        if (res == S_OK && position < length) res = S_FALSE;
    }

    return res;
}

STDMETHODIMP
ArrayEnumPins::Reset() {
    HRESULT res = S_OK;

    {
        m_index = 0;
    }

    return res;
}

STDMETHODIMP
ArrayEnumPins::Clone(IEnumPins **enumPins) {
    HRESULT res = S_OK;

    if (enumPins == NULL) {
        res = E_POINTER;
    } else {
        *enumPins = new ArrayEnumPins(m_pins, m_length);
    }

    return res;
}
