#include "common.hpp"
#include "MediaType.hpp"
EQUIS_USING_NS1(equis);

extern "C" const GUID WMMEDIASUBTYPE_Y800 = {
	MAKEFOURCC('Y', '8', '0', '0'),
	0x0000, 0x0010, 0x80, 0x00, 0x00, 0xAA, 0x00, 0x38, 0x9B, 0x71 };

extern "C" const GUID WMMEDIASUBTYPE_Y160 = {
    MAKEFOURCC('Y', '1', '6', '0'),
	0x0000, 0x0010, 0x80, 0x00, 0x00, 0xAA, 0x00, 0x38, 0x9B, 0x71 };

EQUIS_STATIC_ASSERT(sizeof(MediaType) == sizeof(AM_MEDIA_TYPE));

HRESULT WINAPI CopyMediaType(AM_MEDIA_TYPE* dst, const AM_MEDIA_TYPE* src)
{
    HRESULT res = S_OK;

    if (dst == NULL || src == NULL || (src->cbFormat > 0 && src->pbFormat == NULL)) {
        res = E_POINTER;
    }

    // Copy information to the destination.
    if (res == S_OK) {
        memcpy(dst, src, sizeof(*dst));
        dst->pbFormat = NULL; // Clear the destination pointer; will fix below.
    }

    if (res == S_OK && dst->cbFormat > 0) {
        // Allocate and copy format information for the destination.

        BYTE *format = (BYTE*) CoTaskMemAlloc(dst->cbFormat);
        if (format == NULL) {
            res = E_OUTOFMEMORY;
        } else {
            memcpy(format, src->pbFormat, dst->cbFormat);
            dst->pbFormat = format;
        }
    }

    return res;
}

HRESULT WINAPI CloneMediaType(AM_MEDIA_TYPE** dst, const AM_MEDIA_TYPE* src)
{
    HRESULT res = S_OK;

    if (dst == NULL || src == NULL || (src->cbFormat > 0 && src->pbFormat == NULL)) {
        res = E_POINTER;
    }

    if (SUCCEEDED(res)) {
        AM_MEDIA_TYPE *new_dst = reinterpret_cast<AM_MEDIA_TYPE*>(
            CoTaskMemAlloc(sizeof(*new_dst)));
        if (new_dst == NULL) {
            res = E_OUTOFMEMORY;
        }

        if (SUCCEEDED(res)) {
            res = CopyMediaType(new_dst, src);
        }

        if (SUCCEEDED(res)) {
            *dst = new_dst;
        } else {
            *dst = NULL;
            CoTaskMemFree(new_dst);
        }
    }

    return res;
}

void InitMediaType(AM_MEDIA_TYPE* mt)
{
    memset(mt, 0, sizeof(*mt));
}

void ClearMediaType(AM_MEDIA_TYPE* mt)
{
    if (mt) {
        CoTaskMemFree(mt->pbFormat);
        memset(mt, 0, sizeof(*mt));
    }
}

void DeleteMediaType(AM_MEDIA_TYPE* mt)
{
    if (mt) {
        CoTaskMemFree(mt->pbFormat);
        CoTaskMemFree(mt);
    }
}

static inline bool MatchesGuid(REFGUID filter, REFGUID candidate)
{
    return IsEqualGUID(filter, GUID_NULL) || IsEqualGUID(filter, candidate);
}

bool MatchesMediaType(const AM_MEDIA_TYPE* filter, const AM_MEDIA_TYPE* candidate)
{
    bool matches = true;

    if (filter != NULL) {
        matches &= MatchesGuid(filter->majortype, candidate->majortype);
        matches &= MatchesGuid(filter->subtype, candidate->subtype);
        matches &= filter->bFixedSizeSamples == candidate->bFixedSizeSamples;
        matches &= filter->bTemporalCompression == candidate->bTemporalCompression;
        if (filter->lSampleSize) matches &= filter->lSampleSize == candidate->lSampleSize;
        matches &= MatchesGuid(filter->formattype, candidate->formattype);
    }

    return matches;
}
