#include "common.hpp"
#include "input_spi.h"

static const EVFormat* view_format(EVInputFrame* frame);
static long view_size(EVInputFrame* frame);
static void* view_pointer(EVInputFrame* frame);
static int view_done(EVInputFrame* frame);

static EVInputSpi s_view_spi = {
	NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL, NULL, NULL,

	view_format,
	view_size,
	view_pointer,
	view_done,

	NULL, NULL, NULL, NULL, NULL, NULL
};

static EVInput s_view_input = {
	&s_view_spi
};

struct ViewFrame : public EVInputFrame {
	const EVFormat* format;
	void* data;
	long size;

	ViewFrame(const EVFormat* format, void* data, long size)
		: format(format), data(data), size(size)
	{
		this->origin = &s_view_input;
	}
};

EVInputFrame* evifCreateView(const EVFormat* format, void* data, long size) {
	if (!format || !data) return NULL;

	return new ViewFrame(format, data, size);
}

static const EVFormat* view_format(EVInputFrame* frame) {
	if (!frame) return NULL;
	return static_cast<ViewFrame*>(frame)->format;
}

static long view_size(EVInputFrame* frame) {
	if (!frame) return -1;
	return static_cast<ViewFrame*>(frame)->size;
}

static void* view_pointer(EVInputFrame* frame) {
	if (!frame) return NULL;
	return static_cast<ViewFrame*>(frame)->data;
}

static int view_done(EVInputFrame* frame) {
	if (!frame) return -1;
	delete static_cast<ViewFrame*>(frame);
	return 0;
}
