#include "common.hpp"
#include "xna-common.hpp"
#include "Input_VideoDevice.hpp"

#include "video/dshow/Utility/EnumVideoCaptures.hpp"

#if 0
using Equis::Xna::Input::VideoDevice;
using Equis::Xna::Input::VideoFormat;
using Equis::Xna::Input::VideoFormatPreference;

array<VideoDevice ^> ^VideoDevice::GetAll()
{
	List<VideoDevice ^> ^devices = gcnew List<VideoDevice ^>();
	HRESULT res;

	ComPtr<EnumVideoCaptures> captureEnum;
	captureEnum.AttachWithoutAddRef(new EnumVideoCaptures(res));
	Marshal::ThrowExceptionForHR(res);

	LPCWSTR name = NULL;
	LPCWSTR path = NULL;

	while (true) {
		res = captureEnum->Next();
		if (res == S_FALSE) break;
		Marshal::ThrowExceptionForHR(res);

		res = captureEnum->GetName(&name);
		Marshal::ThrowExceptionForHR(res);

		res = captureEnum->GetPath(&path);
		Marshal::ThrowExceptionForHR(res);

		devices->Add(gcnew VideoDevice(
			Marshal::PtrToStringUni(IntPtr(const_cast<LPWSTR>(name))),
			Marshal::PtrToStringUni(IntPtr(const_cast<LPWSTR>(path)))
			));
	}

	return devices->ToArray();
}

VideoDevice::VideoDevice()
: m_name(nullptr), m_path(nullptr)
{
}

VideoDevice::VideoDevice(String ^name, String ^path)
: m_name(name), m_path(path)
{
}

String ^VideoDevice::Name::get(void) {
	return m_name;
}

String ^VideoDevice::Path::get(void) {
	return m_path;
}

private ref class FormatComparer : IComparer<Equis::Xna::Input::VideoFormat ^>
{
public:
	FormatComparer(array<VideoFormatPreference> ^preferences)
		: m_preferences(preferences)
	{}

	virtual int Compare(VideoFormat ^a, VideoFormat ^b)
	{
		for each (VideoFormatPreference preference in m_preferences)
		{
			int cmp;
			if (preference == VideoFormatPreference::HighFrameRate)
			{
				cmp = int(a->FrameInterval - b->FrameInterval);
			}
			else if (preference == VideoFormatPreference::HighResolution)
			{
				cmp = int(b->Width - a->Width);
			}
			else if (preference == VideoFormatPreference::Monochrome)
			{
				cmp = monochromeFormats->IndexOf(b->Format) - monochromeFormats->IndexOf(a->Format);
			}
			else if (preference == VideoFormatPreference::ManyColors)
			{
				cmp = colorFormats->IndexOf(b->Format) - colorFormats->IndexOf(a->Format);
			}
			else
			{
				throw gcnew ArgumentException("Unsupported video format preference: " + preference.ToString());
			}

			if (cmp != 0) return cmp;
		}
		
		return 0;
	}

private:
	array<VideoFormatPreference> ^m_preferences;
	
	// Ordered from least to greatest, so -1 is worse than all possibilities.
	static initonly List<SurfaceFormat> ^monochromeFormats;
	static initonly List<SurfaceFormat> ^colorFormats;

	static FormatComparer()
	{
		monochromeFormats = gcnew List<SurfaceFormat>();
		monochromeFormats->Add(SurfaceFormat::LuminanceAlpha8);
		monochromeFormats->Add(SurfaceFormat::LuminanceAlpha16);
		monochromeFormats->Add(SurfaceFormat::Luminance8);
		monochromeFormats->Add(SurfaceFormat::Luminance16);

		colorFormats = gcnew List<SurfaceFormat>();
		colorFormats->Add(SurfaceFormat::Bgr555);
		colorFormats->Add(SurfaceFormat::Bgr565);
		colorFormats->Add(SurfaceFormat::Bgr24);
		colorFormats->Add(SurfaceFormat::Bgr32);
		colorFormats->Add(SurfaceFormat::Color);
	}
};

#if 0
array<Equis::Xna::Input::VideoFormat ^> ^VideoDevice::GetFormats(
	array<VideoFormatPreference> ^preferences)
{
	array<VideoFormat ^> ^formats = GetFormats();
	Array::Sort(formats, gcnew FormatComparer(preferences));
	return formats;
}
#endif

#endif